@@ -59,6 +59,8 @@ typedef unsigned char uchar;
 #define VERSION "no-version"
 #endif
 
+int RSKSI_FLAG_TLV16_RUNTIME = RSGT_FLAG_TLV16;
+int RSKSI_FLAG_NONCRIT_RUNTIME = RSGT_FLAG_NONCRIT; 
 
 static void
 reportErr(rsksictx ctx, char *errmsg)
@@ -69,7 +71,7 @@ reportErr(rsksictx ctx, char *errmsg)
 done:	return;
 }
 
-static void
+void
 reportKSIAPIErr(rsksictx ctx, ksifile ksi, char *apiname, int ecode)
 {
 	char errbuf[4096];
@@ -87,31 +89,51 @@ rsksisetErrFunc(rsksictx ctx, void (*func)(void*, uchar *), void *usrptr)
 	ctx->errFunc = func;
 }
 
-imprint_t *
-rsksiImprintFromKSI_DataHash(ksifile ksi, KSI_DataHash *hash)
+int
+rsksiIntoImprintFromKSI_DataHash(imprint_t* imp, ksifile ksi, KSI_DataHash *hash)
 {
-	int r;
-	imprint_t *imp;
+	int r = RSGTE_SUCCESS;
 	const unsigned char *digest;
-	unsigned digest_len;
+	size_t digest_len;
 
-	if((imp = calloc(1, sizeof(imprint_t))) == NULL) {
-		goto done;
-	}
-	int hashID;
+	KSI_HashAlgorithm hashID;
 	r = KSI_DataHash_extract(hash, &hashID, &digest, &digest_len); 
 	if (r != KSI_OK){
 		reportKSIAPIErr(ksi->ctx, ksi, "KSI_DataHash_extract", r);
-		free(imp); imp = NULL; goto done;
+		r = RSGTE_IO; 
+		goto done;
 	}
 
 	imp->hashID = hashID;
 	imp->len = digest_len;
 	if((imp->data = (uint8_t*)malloc(imp->len)) == NULL) {
-		free(imp); imp = NULL; goto done;
+		r = RSGTE_OOM; 
+		goto done;
 	}
 	memcpy(imp->data, digest, digest_len);
-done:	return imp;
+done:	
+	return r;
+}
+
+
+imprint_t *
+rsksiImprintFromKSI_DataHash(ksifile ksi, KSI_DataHash *hash)
+{
+	int r;
+	imprint_t *imp;
+
+	if((imp = calloc(1, sizeof(imprint_t))) == NULL) {
+		goto done;
+	}
+
+	r = rsksiIntoImprintFromKSI_DataHash(imp, ksi, hash); 
+	if (r != RSGTE_SUCCESS) {
+		free(imp); 
+		imp = NULL; 
+		goto done;
+	}
+done:	
+	return imp;
 }
 
 void
@@ -124,25 +146,9 @@ rsksiimprintDel(imprint_t *imp)
 }
 
 int
-rsksiInit(char *usragent)
+rsksiInit(__attribute__((unused)) char *usragent)
 {
-	int r = 0;
-	int ret = KSI_OK;
-
-/*
-	ret = GT_init();
-	if(ret != KSI_OK) {
-		r = 1;
-		goto done;
-	}
-	ret = GTHTTP_init(usragent, 1);
-	if(ret != KSI_OK) {
-		r = 1;
-		goto done;
-	}
-*/
-
-done:	return r;
+	return 0;
 }
 
 void
@@ -154,7 +160,7 @@ rsksiExit(void)
 static inline ksifile
 rsksifileConstruct(rsksictx ctx)
 {
-	ksifile ksi;
+	ksifile ksi = NULL;
 	if((ksi = calloc(1, sizeof(struct ksifile_s))) == NULL)
 		goto done;
 	ksi->ctx = ctx;
@@ -167,14 +173,14 @@ rsksifileConstruct(rsksictx ctx)
 done:	return ksi;
 }
 
-static inline int
+static inline size_t 
 tlvbufPhysWrite(ksifile ksi)
 {
 	ssize_t lenBuf;
 	ssize_t iTotalWritten;
 	ssize_t iWritten;
 	char *pWriteBuf;
-	int r = 0;
+	size_t r = 0;
 
 	lenBuf = ksi->tlvIdx;
 	pWriteBuf = ksi->tlvBuf;
@@ -202,7 +208,7 @@ finalize_it:
 	return r;
 }
 
-static inline int
+static inline size_t 
 tlvbufChkWrite(ksifile ksi)
 {
 	if(ksi->tlvIdx == sizeof(ksi->tlvBuf)) {
@@ -215,19 +221,19 @@ tlvbufChkWrite(ksifile ksi)
 /* write to TLV file buffer. If buffer is full, an actual call occurs. Else
  * output is written only on flush or close.
  */
-static inline int
+static inline size_t
 tlvbufAddOctet(ksifile ksi, int8_t octet)
 {
-	int r;
+	size_t r;
 	r = tlvbufChkWrite(ksi);
 	if(r != 0) goto done;
 	ksi->tlvBuf[ksi->tlvIdx++] = octet;
 done:	return r;
 }
-static inline int
-tlvbufAddOctetString(ksifile ksi, uint8_t *octet, int size)
+static inline size_t 
+tlvbufAddOctetString(ksifile ksi, uint8_t *octet, size_t size)
 {
-	int i, r = 0;
+	size_t i, r = 0;
 	for(i = 0 ; i < size ; ++i) {
 		r = tlvbufAddOctet(ksi, octet[i]);
 		if(r != 0) goto done;
@@ -298,7 +304,7 @@ tlv8WriteKSI(ksifile ksi, int flags, int tlvtype, int len)
 	int r;
 	assert((flags & RSGT_TYPE_MASK) == 0);
 	assert((tlvtype & RSGT_TYPE_MASK) == tlvtype);
-	r = tlvbufAddOctet(ksi, (flags & ~RSGT_FLAG_TLV16) | tlvtype);
+	r = tlvbufAddOctet(ksi, (flags & ~RSKSI_FLAG_TLV16_RUNTIME) | tlvtype);
 	if(r != 0) goto done;
 	r = tlvbufAddOctet(ksi, len & 0xff);
 done:	return r;
@@ -311,7 +317,7 @@ tlv16WriteKSI(ksifile ksi, int flags, int tlvtype, uint16_t len)
 	int r;
 	assert((flags & RSGT_TYPE_MASK) == 0);
 	assert((tlvtype >> 8 & RSGT_TYPE_MASK) == (tlvtype >> 8));
-	typ = ((flags | RSGT_FLAG_TLV16) << 8) | tlvtype;
+	typ = ((flags | RSKSI_FLAG_TLV16_RUNTIME) << 8) | tlvtype;
 	r = tlvbufAddOctet(ksi, typ >> 8);
 	if(r != 0) goto done;
 	r = tlvbufAddOctet(ksi, typ & 0xff);
@@ -334,7 +340,7 @@ tlvWriteHashKSI(ksifile ksi, uint16_t tlvtype, KSI_DataHash *rec)
 	unsigned tlvlen;
 	int r;
 	const unsigned char *digest;
-	unsigned digest_len;
+	size_t digest_len;
 	r = KSI_DataHash_extract(rec, NULL, &digest, &digest_len); 
 	if (r != KSI_OK){
 		reportKSIAPIErr(ksi->ctx, ksi, "KSI_DataHash_extract", r);
@@ -349,6 +355,29 @@ tlvWriteHashKSI(ksifile ksi, uint16_t tlvtype, KSI_DataHash *rec)
 done:	return r;
 }
 
+int
+tlvWriteBlockHdrKSI(ksifile ksi) {
+	unsigned tlvlen;
+	int r;
+	tlvlen  = 2 + 1 /* hash algo TLV */ +
+	 	  2 + hashOutputLengthOctetsKSI(ksi->hashAlg) /* iv */ +
+		  2 + 1 + ksi->x_prev->len /* last hash */;
+	/* write top-level TLV object block-hdr */
+	CHKr(tlv16WriteKSI(ksi, 0x00, 0x0901, tlvlen));
+	/* and now write the children */
+	/* hash-algo */
+	CHKr(tlv8WriteKSI(ksi, 0x00, 0x01, 1));
+	CHKr(tlvbufAddOctet(ksi, hashIdentifierKSI(ksi->hashAlg)));
+	/* block-iv */
+	CHKr(tlv8WriteKSI(ksi, 0x00, 0x02, hashOutputLengthOctetsKSI(ksi->hashAlg)));
+	CHKr(tlvbufAddOctetString(ksi, ksi->IV, hashOutputLengthOctetsKSI(ksi->hashAlg)));
+	/* last-hash */
+	CHKr(tlv8WriteKSI(ksi, 0x00, 0x03, ksi->x_prev->len + 1));
+	CHKr(tlvbufAddOctet(ksi, ksi->x_prev->hashID));
+	CHKr(tlvbufAddOctetString(ksi, ksi->x_prev->data, ksi->x_prev->len));
+done:	return r;
+}
+
 int
 tlvWriteBlockSigKSI(ksifile ksi, uchar *der, uint16_t lenDer)
 {
@@ -357,45 +386,27 @@ tlvWriteBlockSigKSI(ksifile ksi, uchar *der, uint16_t lenDer)
 	int r;
 
 	tlvlenRecords = tlvbufGetInt64OctetSize(ksi->nRecords);
-	tlvlen  = 2 + 1 /* hash algo TLV */ +
-	 	  2 + hashOutputLengthOctetsKSI(ksi->hashAlg) /* iv */ +
-		  2 + 1 + ksi->lenBlkStrtHash /* last hash */ +
-		  2 + tlvlenRecords /* rec-count */ +
+	tlvlen  = 2 + tlvlenRecords /* rec-count */ +
 		  4 + lenDer /* rfc-3161 */;
 	/* write top-level TLV object (block-sig */
-	r = tlv16WriteKSI(ksi, 0x00, 0x0902, tlvlen);
+	r = tlv16WriteKSI(ksi, 0x00, 0x0904, tlvlen);
 	if(r != 0) goto done;
 	/* and now write the children */
 	//FIXME: flags???
-	/* hash-algo */
-	r = tlv8WriteKSI(ksi, 0x00, 0x00, 1);
-	if(r != 0) goto done;
-	r = tlvbufAddOctet(ksi, hashIdentifierKSI(ksi->hashAlg));
-	if(r != 0) goto done;
-	/* block-iv */
-	r = tlv8WriteKSI(ksi, 0x00, 0x01, hashOutputLengthOctetsKSI(ksi->hashAlg));
-	if(r != 0) goto done;
-	r = tlvbufAddOctetString(ksi, ksi->IV, hashOutputLengthOctetsKSI(ksi->hashAlg));
-	if(r != 0) goto done;
-	/* last-hash */
-	r = tlv8WriteKSI(ksi, 0x00, 0x02, ksi->lenBlkStrtHash+1);
-	if(r != 0) goto done;
-	r = tlvbufAddOctet(ksi, hashIdentifierKSI(ksi->hashAlg));
-	if(r != 0) goto done;
-	r = tlvbufAddOctetString(ksi, ksi->blkStrtHash, ksi->lenBlkStrtHash);
-	if(r != 0) goto done;
 	/* rec-count */
-	r = tlv8WriteKSI(ksi, 0x00, 0x03, tlvlenRecords);
+	r = tlv8WriteKSI(ksi, 0x00, 0x01, tlvlenRecords);
 	if(r != 0) goto done;
 	r = tlvbufAddInt64(ksi, ksi->nRecords);
 	if(r != 0) goto done;
-	/* rfc-3161 */
-	r = tlv16WriteKSI(ksi, 0x00, 0x906, lenDer);
+	/* Open-KSI signature */
+	r = tlv16WriteKSI(ksi, 0x00, 0x0905, lenDer);
 	if(r != 0) goto done;
 	r = tlvbufAddOctetString(ksi, der, lenDer);
 done:	return r;
 }
 
+
+
 /* support for old platforms - graceful degrade */
 #ifndef O_CLOEXEC
 #define O_CLOEXEC 0
@@ -417,19 +428,31 @@ readStateFile(ksifile ksi)
 	if(read(fd, &sf, sizeof(sf)) != sizeof(sf)) goto err;
 	if(strncmp(sf.hdr, "KSISTAT10", 9)) goto err;
 
-	ksi->lenBlkStrtHash = sf.lenHash;
-	ksi->blkStrtHash = calloc(1, ksi->lenBlkStrtHash);
-	if(read(fd, ksi->blkStrtHash, ksi->lenBlkStrtHash)
-		!= ksi->lenBlkStrtHash) {
-		free(ksi->blkStrtHash);
+	ksi->x_prev = malloc(sizeof(imprint_t));
+	if (ksi->x_prev == NULL) goto err;
+	ksi->x_prev->len = sf.lenHash;
+	ksi->x_prev->hashID = sf.hashID;
+	ksi->x_prev->data = calloc(1, ksi->x_prev->len);
+	if (ksi->x_prev->data == NULL) {
+		free(ksi->x_prev);
+		ksi->x_prev = NULL;
+		goto err;
+	}
+
+	if(read(fd, ksi->x_prev->data, ksi->x_prev->len)
+		!= (ssize_t) ksi->x_prev->len) {
+		rsksiimprintDel(ksi->x_prev);
+		ksi->x_prev = NULL;
 		goto err;
 	}
 	close(fd);
 return;
 
 err:
-	ksi->lenBlkStrtHash = hashOutputLengthOctetsKSI(ksi->hashAlg);
-	ksi->blkStrtHash = calloc(1, ksi->lenBlkStrtHash);
+	ksi->x_prev = malloc(sizeof(imprint_t));
+	ksi->x_prev->hashID = hashIdentifierKSI(ksi->hashAlg);
+	ksi->x_prev->len = hashOutputLengthOctetsKSI(ksi->hashAlg);
+	ksi->x_prev->data = calloc(1, ksi->x_prev->len);
 }
 
 /* persist all information that we need to re-open and append
@@ -561,6 +584,8 @@ rsksiCtxOpenFile(rsksictx ctx, unsigned char *logfn)
 	ksi->statefilename = (uchar*) strdup(fn);
 	if(tlvOpenKSI(ksi, LOGSIGHDR, sizeof(LOGSIGHDR)-1) != 0) {
 		reportErr(ctx, "signature file open failed");
+		/* Free memory */
+		free(ksi);
 		ksi = NULL;
 	}
 done:	return ksi;
@@ -584,8 +609,6 @@ rsksiSetHashFunction(rsksictx ctx, char *algName)
 		ctx->hashAlg = KSI_HASHALG_SHA2_384;
 	else if(!strcmp(algName, "SHA2-512"))
 		ctx->hashAlg = KSI_HASHALG_SHA2_512;
-	else if(!strcmp(algName, "RIPEMD-256"))
-		ctx->hashAlg = KSI_HASHALG_RIPEMD_256;
 	else if(!strcmp(algName, "SHA3-244"))
 		ctx->hashAlg = KSI_HASHALG_SHA3_244;
 	else if(!strcmp(algName, "SHA3-256"))
@@ -617,7 +640,6 @@ rsksifileDestruct(ksifile ksi)
 	free(ksi->sigfilename);
 	free(ksi->statefilename);
 	free(ksi->IV);
-	free(ksi->blkStrtHash);
 	rsksiimprintDel(ksi->x_prev);
 	free(ksi);
 done:	return r;
@@ -639,7 +661,7 @@ sigblkInitKSI(ksifile ksi)
 {
 	if(ksi == NULL) goto done;
 	seedIVKSI(ksi);
-	memset(ksi->roots_valid, 0, sizeof(ksi->roots_valid)/sizeof(char));
+	memset(ksi->roots_valid, 0, sizeof(ksi->roots_valid));
 	ksi->nRoots = 0;
 	ksi->nRecords = 0;
 	ksi->bInBlk = 1;
@@ -651,35 +673,32 @@ done:	return;
 static inline void
 bufAddIV(ksifile ksi, uchar *buf, size_t *len)
 {
-	memcpy(buf+*len, ksi->IV, hashOutputLengthOctetsKSI(ksi->hashAlg));
-	*len += sizeof(ksi->IV);
+	int hashlen;
+
+	hashlen = hashOutputLengthOctetsKSI(ksi->hashAlg);
+
+	memcpy(buf+*len, ksi->IV, hashlen);
+	*len += hashlen;
 }
 
 
 /* concat: add imprint to buffer */
 static inline void
-bufAddImprint(ksifile ksi, uchar *buf, size_t *len, imprint_t *imp)
+bufAddImprint(uchar *buf, size_t *len, imprint_t *imp)
 {
-	if(imp == NULL) {
-	/* TODO: how to get the REAL HASH ID? --> add field? */
-		buf[*len] = hashIdentifierKSI(ksi->hashAlg);
-		++(*len);
-		memcpy(buf+*len, ksi->blkStrtHash, ksi->lenBlkStrtHash);
-		*len += ksi->lenBlkStrtHash;
-	} else {
-		buf[*len] = imp->hashID;
-		++(*len);
-		memcpy(buf+*len, imp->data, imp->len);
-		*len += imp->len;
-	}
+	buf[*len] = imp->hashID;
+	++(*len);
+	memcpy(buf+*len, imp->data, imp->len);
+	*len += imp->len;
 }
+
 /* concat: add hash to buffer */
 static inline void
 bufAddHash(ksifile ksi, uchar *buf, size_t *len, KSI_DataHash *hash)
 {
 	int r; 
 	const unsigned char *digest;
-	unsigned digest_len;
+	size_t digest_len;
 	r = KSI_DataHash_extract(hash, NULL, &digest, &digest_len); // TODO: error check
 	if (r != KSI_OK){
 		reportKSIAPIErr(ksi->ctx, ksi, "KSI_DataHash_extract", r);
@@ -708,7 +727,7 @@ hash_m_ksi(ksifile ksi, KSI_DataHash **m)
 	size_t len = 0;
 	int r = 0;
 
-	bufAddImprint(ksi, concatBuf, &len, ksi->x_prev);
+	bufAddImprint(concatBuf, &len, ksi->x_prev);
 	bufAddIV(ksi, concatBuf, &len);
 	rgt = KSI_DataHash_create(ksi->ctx->ksi_ctx, concatBuf, len, ksi->hashAlg, m);
 	if(rgt != KSI_OK) {
@@ -765,12 +784,14 @@ sigblkAddRecordKSI(ksifile ksi, const uchar *rec, const size_t len)
 	if(ksi == NULL || ksi->disabled) goto done;
 	if((ret = hash_m_ksi(ksi, &m)) != 0) goto done;
 	if((ret = hash_r_ksi(ksi, &r, rec, len)) != 0) goto done;
+	if(ksi->nRecords == 0) 
+		tlvWriteBlockHdrKSI(ksi);
 	if(ksi->bKeepRecordHashes)
-		tlvWriteHashKSI(ksi, 0x0900, r);
+		tlvWriteHashKSI(ksi, 0x0902, r);
 	if((ret = hash_node_ksi(ksi, &x, m, r, 1)) != 0) goto done; /* hash leaf */
 	/* persists x here if Merkle tree needs to be persisted! */
 	if(ksi->bKeepTreeHashes)
-		tlvWriteHashKSI(ksi, 0x0901, x);
+		tlvWriteHashKSI(ksi, 0x0903, x);
 	rsksiimprintDel(ksi->x_prev);
 	ksi->x_prev = rsksiImprintFromKSI_DataHash(ksi, x);
 	/* add x to the forest as new leaf, update roots list */
@@ -790,7 +811,7 @@ sigblkAddRecordKSI(ksifile ksi, const uchar *rec, const size_t len)
 			KSI_DataHash_free(t_del);
 			if(ret != 0) goto done;
 			if(ksi->bKeepTreeHashes)
-				tlvWriteHashKSI(ksi, 0x0901, t);
+				tlvWriteHashKSI(ksi, 0x0903, t);
 		}
 	}
 	if(t != NULL) {
@@ -823,7 +844,7 @@ static int
 signIt(ksifile ksi, KSI_DataHash *hash)
 {
 	unsigned char *der = NULL;
-	unsigned lenDer;
+	size_t lenDer;
 	int r = KSI_OK;
 	int ret = 0;
 	KSI_Signature *sig = NULL;
@@ -890,10 +911,6 @@ sigblkFinishKSI(ksifile ksi)
 	if((ret = signIt(ksi, root)) != 0) goto done;
 
 	KSI_DataHash_free(root);
-	free(ksi->blkStrtHash);
-	ksi->lenBlkStrtHash = ksi->x_prev->len;
-	ksi->blkStrtHash = malloc(ksi->lenBlkStrtHash);
-	memcpy(ksi->blkStrtHash, ksi->x_prev->data, ksi->x_prev->len);
 done:
 	ksi->bInBlk = 0;
 	return ret;
