@@ -735,7 +735,7 @@ verifyKSI(char *name, char *errbuf, char *sigfname, char *oldsigfname, char *nsi
 	int FILEMODE = FILEMODE_LOGSIG; /* Default FileMode */ 
 	block_sig_t *bs = NULL;
 	block_hdr_t *bh = NULL;
-	ksifile ksi;
+	ksifile ksi = NULL;
 	uint8_t bHasRecHashes, bHasIntermedHashes;
 	uint8_t bInBlock;
 	int r = 0;
@@ -764,7 +764,7 @@ verifyKSI(char *name, char *errbuf, char *sigfname, char *oldsigfname, char *nsi
 	}
 
 	/* Check if we have a logsignature file */
-	if((r = rsksi_chkFileHdr(sigfp, "LOGSIG11", verbose)) == 0) {
+	if((r = rsksi_chkFileHdr(sigfp, "LOGSIG11", 0)) == 0) {
 		/* Verify Log signature */
 		if(debug) printf("debug: verifyKSI:\t\t\t Found log signature file ... \n");
 		if(mode == MD_EXTEND) {
@@ -814,7 +814,6 @@ verifyKSI(char *name, char *errbuf, char *sigfname, char *oldsigfname, char *nsi
 		if(debug) printf("verifyKSI:\t\t\t Found record integrity proof file ... \n");
 		FILEMODE = FILEMODE_RECSIG; 
 
-		bInBlock = 0;
 		ectx.blkNum = 0;
 		ectx.recNumInFile = 0;
 
@@ -896,8 +895,10 @@ if (debug) printf("debug: verifyKSI:\t\t\t DONE with LOOP iCurrentLine=%d > iMax
 	}
 done:
 	/* Free mem first */
-	if (lineRec != NULL)
+	if (lineRec != NULL) {
 		free(lineRec);
+		lineRec = NULL; 
+	}
 
 	if(r != RSGTE_EOF)
 		goto err;
@@ -997,7 +998,7 @@ extractKSI(char *name, char *errbuf, char *sigfname, FILE *logfp, FILE *sigfp)
 	block_sig_t *bs = NULL;
 	block_hdr_t *bh = NULL;
 	tlvrecord_t tlvbhrec; 
-	ksifile ksi;
+	ksifile ksi = NULL;
 	uint8_t bHasRecHashes, bHasIntermedHashes;
 	uint8_t bInBlock;
 	int bInitDone = 0;
@@ -1028,10 +1029,11 @@ extractKSI(char *name, char *errbuf, char *sigfname, FILE *logfp, FILE *sigfp)
 		char* pszTmp = linenumbers;
 		if (*(pszTmp) != ',')
 			iLineNumbers++; 
+		else
+			pszTmp++; /*Next Char*/
 
 		while(pszTmp != NULL) {
 			pszTmp = strchr(pszTmp, ',');
-
 			if( pszTmp != NULL) {
 				pszTmp++;
 				if ( *(pszTmp) == ',' ) {
@@ -1042,6 +1044,11 @@ extractKSI(char *name, char *errbuf, char *sigfname, FILE *logfp, FILE *sigfp)
 				}
 			}
 		}
+		if (iLineNumbers == 0) {
+			fprintf(stderr, "extractKSI:\t\t\t error invalid linenumbers\n");
+			r = RSGTE_IO;
+			goto done;
+		}
 		if (debug) printf("debug: extractKSI:\t\t\t found '%d' linenumbers\n", iLineNumbers);
 
 		/* Convert line numbers into int Array */
@@ -1050,7 +1057,7 @@ extractKSI(char *name, char *errbuf, char *sigfname, FILE *logfp, FILE *sigfp)
 		int iNumLength = 0; 
 		char szTmpNum[11]; 
 		char* pszBegin = linenumbers;
-		char* pszEnd = linenumbers;
+		char* pszEnd; 
 		while(pszBegin != NULL) {
 			/* Cut number from string */
 			pszEnd = strchr(pszBegin, ',');
@@ -1160,11 +1167,8 @@ extractKSI(char *name, char *errbuf, char *sigfname, FILE *logfp, FILE *sigfp)
 	{
 		/* Free previous hashchain */
 		if (hashchain != NULL) {
-			if (hashchain->rec_hash.data == NULL) free(hashchain->rec_hash.data); 
-			for(j = 0 ; j < hashchain->stepCount ; ++j) {
-				if (hashchain->hashsteps[j]->sib_hash.data == NULL) free(hashchain->hashsteps[j]->sib_hash.data); 
-			}
-			free(hashchain); 
+			rsksi_objfree(0x0907, hashchain); 
+			hashchain = NULL;
 		}
 
 		/* Init new HashChain */
@@ -1245,8 +1249,8 @@ if (debug) printf("debug: extractKSI:\t\t\t line '%d': %.64s...\n", iLineSearch,
 			/* Check if this is a new signature block */
 			if(bInBlock == 0) {
 				/* Free memory */
-				if(bs != NULL) rsksi_objfree(0x0904, bs);
-				if(bh != NULL) rsksi_objfree(0x0901, bh);
+				if(bs != NULL) { rsksi_objfree(0x0904, bs); bs = NULL; }
+				if(bh != NULL) { rsksi_objfree(0x0901, bh); bh = NULL; }
 				
 				/* Get/Verify Block Paramaters */
 				if((r = rsksi_getBlockParams(ksi, sigfp, 1, &bs, &bh, &bHasRecHashes, &bHasIntermedHashes)) != 0) {
@@ -1351,6 +1355,24 @@ done:
 	if (paiLineNumbers != NULL)
 		free(paiLineNumbers); 
 
+	/* Free hashchain */
+	if (hashchain != NULL) {
+		rsksi_objfree(0x0907, hashchain); 
+		hashchain = NULL;
+	}
+
+	/* Free Blockheader / Sig */
+	if(bs != NULL) rsksi_objfree(0x0904, bs);
+	if(bh != NULL) rsksi_objfree(0x0901, bh);
+
+	/* Free KSI File helper stuff */
+	if (ksi != NULL) {
+		rsksiCtxDel(ksi->ctx);
+		free(ksi->IV);
+		rsksiimprintDel(ksi->x_prev);
+		free(ksi);
+	}
+
 	if(r != RSGTE_EOF) {
 		goto done2;
 	}
@@ -1677,7 +1699,7 @@ rsgtutil_usage(void)
 			"\t-B, --show-sigblock-params \t Show signature block parameters.\n"
 			"\t-T, --detect-file-type \t Show Type of signature file.\n"
 			"\t-c, --convert \t\t\t Convert Signature Format Version 10 to 11.\n"
-			"\t-V, --Version \t\t\t Print utility version\n"
+			"\t-V, --version \t\t\t Print utility version\n"
 			"\t\tOptional parameters\n"
 			"\t-a <GT|KSI>, --api  <GT|KSI> \t Set which API to use.\n"
 			"\t\tGT = Guardtime Client Library\n"
@@ -1701,7 +1723,7 @@ main(int argc, char *argv[])
 	int opt;
 
 	while(1) {
-		opt = getopt_long(argc, argv, "aABcdDeEHkoPstTuvVx", long_options, NULL);
+		opt = getopt_long(argc, argv, "a:ABcdDeE:hk:o:P:stTu:vVx:", long_options, NULL);
 		if(opt == -1)
 			break;
 		switch(opt) {
