@@ -3,18 +3,18 @@
  * because it was either written from scratch by me (rgerhards) or
  * contributors who agreed to ASL 2.0.
  *
- * Copyright 2004-2015 Rainer Gerhards and Adiscon
+ * Copyright 2004-2016 Rainer Gerhards and Adiscon
  *
  * This file is part of rsyslog.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *       http://www.apache.org/licenses/LICENSE-2.0
  *       -or-
  *       see COPYING.ASL20 in the source distribution
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -1494,6 +1494,7 @@ static void
 mainloop(void)
 {
 	struct timeval tvSelectTimeout;
+	time_t tTime;
 
 	BEGINfunc
 	/* first check if we have any internal messages queued and spit them out. */
@@ -1518,6 +1519,9 @@ mainloop(void)
 
 		janitorRun();
 
+		datetime.GetTime(&tTime);
+		checkGoneAwaySenders(tTime);
+
 		if(bHadHUP) {
 			doHUP();
 			bHadHUP = 0;
@@ -1632,6 +1636,8 @@ deinitAll(void)
 int
 main(int argc, char **argv)
 {
+	/* use faster hash function inside json lib */
+	json_global_set_string_hash(JSON_C_STR_HASH_PERLLIKE);
 	dbgClassInit();
 	initAll(argc, argv);
 	sd_notify(0, "READY=1");
